# Tests the functions used to implement the pipevis-listener app

from __future__ import division
import unittest
import Queue
import threading
import sys
import traceback

from netsa.data.times import make_datetime


from pipevis import (
    MainLoop,
    Mother,
    AlertSource,
    SqlWriter,
    ListenerApp,
)

from fake import *

import logging


class FakeAlertModule(FakeObject):
    def subscribe(self, channel_name, callback):
        self._callback = callback
        self.call_method('subscribe', channel_name, callback)
    def unsubscribe(self, channel_name):
        self.call_method('unsubscribe', channel_name, callback)
    def test_send_alert(self, alert):
        self._callback(alert)


class FakeAlertSource(FakeObject):
    def test_configure(self, origin, channel, callback):
        # Don't really need origin or channel for this test
        self._callback = callback
    def test_inject(self, alert):
        self._callback(alert)
    def start(self):
        self.call_method('start')
    def stop(self):
        self.call_method('stop')


class FakeMother(Mother):
    # Override some methods to give instrumented versions
    def __init__(self, alert_source, writer, *args, **karghs):
        Mother.__init__(self, *args, **karghs)
        self._alert_source = alert_source
        self._writer = writer
    def new_alert_source(self, origin, channel, callback):
        # Note that this has nothing to do with how the actual alert
        # source is configured. See alertsource.py for that.
        self._alert_source.test_configure(origin=origin,
                                          channel=channel,
                                          callback=callback)
        return self._alert_source
    def new_sql_writer(self, connection_uri, bin_size):
        return self._writer

# Stolen from test_loop. (TODO: refactor into common function?)

class LoopThread(threading.Thread):
    def __init__(self, loop):
        threading.Thread.__init__(self)
        self._loop = loop
        self._exception = None
    def run(self):
        try:
            # This is different from the test_loop thread -- we have
            # to call app.run() instead
            self._loop.run()
        except BaseException, e:
            t, v, tb = sys.exc_info()
            self._exception = (t, v, tb, e)
    def get_exception(self):
        if self._exception is None:
            return None
        else:
            return self._exception[3]
    def print_traceback(self):
        traceback.print_exception(*self._exception[:3])
    
def start_loop_thread(loop):
    loop_thread = LoopThread(loop=loop)
    loop_thread.setDaemon(True) # So we can exit -- this may leave zombies
    loop_thread.start()
    return loop_thread



# Test that once listenapp is running, it takes alerts from the sink
# and gives them to the writer.


class TestListenerApp(unittest.TestCase):
    def test_basic(self):
        def event_and_handler(custom_handling=None):
            evt = threading.Event()
            def handler(*args, **kargs):
                if custom_handling is not None:
                    custom_handling(*args, **kargs)
                evt.set()
            return (evt, handler)

        exp_timestamp = make_datetime("2001/01/01:02:34:56")
        exp_alert = generate_default_alert(timestamp=exp_timestamp)

        valid_alert = [None]
        def validate_alert(alert):
            valid_alert[0] = alert


        evt_start, on_start = event_and_handler()
        # TODO: test stop
        evt_alert_written, on_alert_write = event_and_handler(
            custom_handling=validate_alert)            

        alert_source = FakeAlertSource(start=on_start)

        mother = FakeMother(alert_source=alert_source,
                            writer=FakeWriter(__call__=on_alert_write))

        def wait_on_event(evt, evt_name, timeout=.75):
            evt.wait(timeout=timeout)
            self.assertTrue(evt.isSet(), "'%s' didn't happen" % evt_name)

        opts = dict(channel="foo",
                    dst="bar",
                    origin="baz",
                    poll_sleep="500")

        app = ListenerApp(opts=opts, mother=mother)
        loop_thread = start_loop_thread(app)

        wait_on_event(evt_start, 'start')
        alert_source.test_inject(exp_alert)
        wait_on_event(evt_alert_written, 'alert_written')
        self.assertEquals(id(exp_alert), id(valid_alert[0]))
                            


if __name__ == '__main__':
    unittest.main()
